/***************************************************************************\
  "memtrack.h" - Memory Tracker For Debugging (Global)
  Revision 3; 16 Jan 2003 - 9 Feb 2004
  Copyright (C) 2003-2004  Kevin P. Albrecht
\***************************************************************************/

#ifndef MEMTRACK_H
#define MEMTRACK_H

#ifdef GLOBAL_DEBUG

  /////////////////////////
  // DEFINES //////////////
  /////////////////////////

  // External Defines //
  // GLOBAL_DEBUG or GLOBAL_RELEASE

  /////////////////////////
  // INCLUDES /////////////
  /////////////////////////

  // Standard //

  // Local //

  /////////////////////////
  // DECLARATIONS /////////
  /////////////////////////

  typedef struct MemUnit_tag
  {
    char *file;
    int line;
    void *pointer;
    struct MemUnit_tag* next;
    struct MemUnit_tag* prev;
  } MemUnit;

  MemUnit *memtrack_head;
  unsigned int memtrack_units;
  unsigned int memtrack_units_total;

  void memtrack_init ();
  void memtrack_deinit ();
  void *memtrack_jalloc (unsigned int size, const char *file, int line);
  void memtrack_jfree (void *pointer, const char *file, int line);

  /////////////////////////
  // DEFINITIONS //////////
  /////////////////////////

  void memtrack_init ()
  {
    memtrack_head = NULL;
    memtrack_units = 0;
    memtrack_units_total = 0;
  }

  void memtrack_deinit ()
  {
    MemUnit *temp;

    temp = memtrack_head;

    printf   ("**MEMTRACK:COMPLETED** %d memory leak(s) found, %d total units allocatated\n", memtrack_units, memtrack_units_total);

    while (memtrack_head != NULL)
    {
      memtrack_head = temp->next;

      printf ("**MEMTRACK:LEAK** %s(%d), ptr<0x%x>, str<%s>\n", temp->file, temp->line, (int)temp->pointer, (char*)(temp->pointer));

      free (temp->file);
      //NOTE: temp->pointer not deleted here because it should be deleted in the program.
      free (temp);
      temp = memtrack_head;
    }
  }

  void *memtrack_jalloc (unsigned int size, const char *file, int line)
  {
    MemUnit *temp;
    int n;

    temp = (MemUnit *) malloc (sizeof (MemUnit));

    n = strlen (file) * sizeof (char);
    temp->file = (char *) malloc (n + 1);
    strcpy (temp->file, file);
    temp->line = line;
    temp->pointer = malloc (size);
    temp->next = memtrack_head;
    temp->prev = NULL;
    if (memtrack_head) memtrack_head->prev = temp;
    memtrack_head = temp;

    memtrack_units++;
    memtrack_units_total++;

    return temp->pointer;
  }

  void memtrack_jfree (void *pointer, const char *file, int line)
  {
    MemUnit *curr;

    if (pointer == NULL) return;

    curr = memtrack_head;

    while (curr != NULL)
    {
      if (curr->pointer == pointer)
      {
        free (curr->pointer);
        free (curr->file);

        if (curr->prev == NULL)
        {
          memtrack_head = curr->next;
          if (memtrack_head) memtrack_head->prev = NULL;
        }
        else
        {
          curr->prev->next = curr->next;
          if (curr->next) curr->next->prev = curr->prev;
        }

        free (curr);
        memtrack_units--;        
        return;
      }

      curr = curr->next;
    }

    printf ("**CANT_FREE**: %s(%d), ptr<0x%x>, str<%s>\n", file, line, (int)pointer, (char*)(pointer));
  }

#endif

#ifdef GLOBAL_DEBUG
  #define JALLOC(size) memtrack_jalloc((size), __FILE__, __LINE__)
#else
  #define JALLOC(size) malloc((size))
#endif

#ifdef GLOBAL_DEBUG
  #define JFREE(pointer) memtrack_jfree((pointer), __FILE__, __LINE__)
#else
  #define JFREE(pointer) free(pointer)
#endif

#endif
