/***************************************************************************\
  "rstack.h" - Runtime Stack (Phase E)
  Revision 2; 27 Nov 2002 - 16 Jan 2003
  Copyright (C) 2002-2003  Kevin P. Albrecht
\***************************************************************************/

#ifndef RSTACK_H
#define RSTACK_H

/////////////////////////
// DEFINES //////////////
/////////////////////////

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //
#include <stdio.h>

// Local //
#include "global.h"
#include "error.h"
#include "table.h"
#include "global_conv.h"

/////////////////////////
// DECLARATIONS /////////
/////////////////////////

// External //
typedef struct
{
  Symbol *top;
  int count;
} RStack;

RStack *rstack_new ();
void rstack_del (RStack *rs);
void rstack_push (RStack *rs, Symbol *sym);
void *rstack_pop (RStack *rs);
Type rstack_peektype (RStack *rs);

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

RStack *rstack_new ()
{
  RStack *rs = (RStack*) JALLOC(sizeof (RStack));
  rs->top = NULL;
  rs->count = 0;
  return rs;
}

void rstack_del (RStack *rs)
{
  Symbol *temp;

  while (rs->top != NULL)
  {
    temp = rs->top;
    rs->top = rs->top->next;
    sym_del (temp);
  }
  
  JFREE(rs);
}

void rstack_push (RStack *rs, Symbol *sym)
{
  Symbol *temp;

  if (rs->top == NULL)
  {
    rs->top = sym_copy (sym);
    rs->top->next = NULL;
  }
  else
  {
    temp = rs->top;
    rs->top = sym_copy (sym);
    rs->top->next = temp;
  }

  rs->count++;
}

void *rstack_pop (RStack *rs)
{
  Symbol *symtemp;
  void *vtemp;

  if (rs->top != NULL)
  {
    symtemp = rs->top;
    rs->top = rs->top->next;
    rs->count--;
    vtemp = symtemp->value;
    sym_del_savevalue (symtemp);
    return vtemp;
  }
  else
  {
    return NULL;
  }
}

Type rstack_peektype (RStack *rs)
{
  if (rs->top != NULL) return rs->top->type;
  else                 return tNull;
}

#endif

