/***************************************************************************\
  "token.h" - Token Variables and Functions (Phase B)
  Revision 17; 10 Nov 2002 - 4 Feb 2004
  Copyright (C) 2002-2004  Kevin P. Albrecht
\***************************************************************************/

#ifndef TOKEN_H
#define TOKEN_H

/////////////////////////
// DEFINES //////////////
/////////////////////////

// External Defines //
// GLOBAL_TOKEN_SIZE

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //

// Local //
#include "global.h"

/////////////////////////
// DECLARATIONS /////////
/////////////////////////

typedef enum
{
  // null //
  codeNull, codeError,

  // codeStart is passed to lex_get at the start of the file //
  codeStart,

  // valued token types //
  codeIdentifier, codeIntLiteral, codeRatLiteral, codeStrLiteral,

  // source code whitespace types //
  codeEOF, codeEOL,

  // reserved words //
  codeOR,
  codeAND, codeDEF, codeINT, codeRAT, codeSTR, codePOP, codeMOD,
  codeFILE, codeGOTO, codeCOND, codeLOOP, codePUSH, codeROUT,
  codeCONST, 
  codeRETURN,
  code_PRIM_CIN, code_PRIM_FIO,  code_PRIM_ASC,
  code_PRIM_COUT, code_PRIM_RAND, code_PRIM_STOP,

  // specials //
  codePLUS, codeMINUS, codeTIMES, codeFORWARDSLASH,
  codeBACKSLASH, codeEQUALS, codeLEFTPAREN, codeRIGHTPAREN,
  codeCOMMA, codeLT, codeGT, codeDOLLAR,
  codeDOT, codeLEFTBRACK, codeRIGHTBRACK, codeQUOTEMARK,
  codeSEMICOLON, codeCOLON, codeNOTEQUAL, codeLTEQUAL,
  codeGTEQUAL, codeLEFTBRACE, codeRIGHTBRACE, codeANDSIGN,
  codePOUNDSIGN, codeARROW, codeEXPONENT
} TokenCode;

char *gvsTokenCodeStrings[] =
{
  // null //
  "<null>", "<error>",

  // codeStart is passed to lex_get at the start of the file //    
  "<start>",

  // valued token types //
  "<identifier>", "<int_literal>", "<rat_literal>", "<str_literal>", 

  // source code whitespace types //
  "<eof>", "<eol>",

  // reserved words //
  "or",
  "and", "def", "int", "rat", "str", "pop", "mod",
  "file", "goto", "cond", "loop", "push", "rout",
  "const",
  "return",
  "_prim_cin", "_prim_fio", "_prim_asc",
  "_prim_cout", "_prim_rand", "_prim_stop",

  // specials //
  "+", "-", "*", "/",
  "\\", "=", "(", ")",
  ",", "<", ">", "$",
  ".", "[", "]", "\"",
  ";", ":", "<>", "<=",
  ">=", "{", "}", "&",
  "#", "<-", "^"
};

typedef struct
{
  TokenCode code;
  char lexeme[GLOBAL_TOKEN_SIZE]; // the actual text of the token as it appears in input
  void *value; // the value held in the token:
               // CODE            VALUE          MEANING
               // ----            -----          -------
               // codeIntLiteral  int*           integer
               // codeRatLiteral  float*         rational
               // codeStrLiteral  char*          string
               // (other)         void*          (none)
} Token;

void tok_clear (Token *token);
Token *tok_new ();
void tok_del (Token *token);
void tok_set (Token *token, void *aValue);

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

void tok_clear (Token *token)
{
  token->code = codeNull;
  k_copy (token->lexeme, "");
  token->value = NULL;
}

void tok_del (Token *token)
{
  // token may be NULL //
  if (!token) return;

  if (token->code == codeStrLiteral)
  {
    if (((char*)(token->value))[0] != 0) // this is a bug fix so that deleting an empty string won't cause an error.
      JFREE(token->value);
  }
  else
  {
    JFREE(token->value);
  }

  JFREE(token);
}

Token *tok_new ()
{
  Token *temp = (Token *) JALLOC(sizeof (Token));
  tok_clear (temp);
  return temp;
}

void tok_set (Token *token, void *aValue)
{
  token->value = aValue;
}

#endif
